from pandas.core.base import PandasObject as PandasObject
from pandas.io.formats.printing import pprint_thing as pprint_thing
from typing import Any, List


class FrozenList(PandasObject, List[Any]):
    def union(self, other: Any) -> FrozenList: ...
    def difference(self, other: Any) -> FrozenList: ...
    def __radd__(self, other: Any) -> Any: ...
    __req__: Any = ...
    def __mul__(self, other: Any) -> Any: ...
    __imul__: Any = ...
    pop: Any = ...
    append: Any = ...
    extend: Any = ...
    remove: Any = ...
    sort: Any = ...
    insert: Any = ...



from typing import Any

magic: Any
align_1_checker_value: bytes
align_1_offset: int
align_1_length: int
align_1_value: int
u64_byte_checker_value: bytes
align_2_offset: int
align_2_length: int
align_2_value: int
endianness_offset: int
endianness_length: int
platform_offset: int
platform_length: int
encoding_offset: int
encoding_length: int
dataset_offset: int
dataset_length: int
file_type_offset: int
file_type_length: int
date_created_offset: int
date_created_length: int
date_modified_offset: int
date_modified_length: int
header_size_offset: int
header_size_length: int
page_size_offset: int
page_size_length: int
page_count_offset: int
page_count_length: int
sas_release_offset: int
sas_release_length: int
sas_server_type_offset: int
sas_server_type_length: int
os_version_number_offset: int
os_version_number_length: int
os_maker_offset: int
os_maker_length: int
os_name_offset: int
os_name_length: int
page_bit_offset_x86: int
page_bit_offset_x64: int
subheader_pointer_length_x86: int
subheader_pointer_length_x64: int
page_type_offset: int
page_type_length: int
block_count_offset: int
block_count_length: int
subheader_count_offset: int
subheader_count_length: int
page_meta_type: int
page_data_type: int
page_amd_type: int
page_metc_type: int
page_comp_type: int
page_mix_types: Any
subheader_pointers_offset: int
truncated_subheader_id: int
compressed_subheader_id: int
compressed_subheader_type: int
text_block_size_length: int
row_length_offset_multiplier: int
row_count_offset_multiplier: int
col_count_p1_multiplier: int
col_count_p2_multiplier: int
row_count_on_mix_page_offset_multiplier: int
column_name_pointer_length: int
column_name_text_subheader_offset: int
column_name_text_subheader_length: int
column_name_offset_offset: int
column_name_offset_length: int
column_name_length_offset: int
column_name_length_length: int
column_data_offset_offset: int
column_data_length_offset: int
column_data_length_length: int
column_type_offset: int
column_type_length: int
column_format_text_subheader_index_offset: int
column_format_text_subheader_index_length: int
column_format_offset_offset: int
column_format_offset_length: int
column_format_length_offset: int
column_format_length_length: int
column_label_text_subheader_index_offset: int
column_label_text_subheader_index_length: int
column_label_offset_offset: int
column_label_offset_length: int
column_label_length_offset: int
column_label_length_length: int
rle_compression: bytes
rdc_compression: bytes
compression_literals: Any
encoding_names: Any

class SASIndex:
    row_size_index: int = ...
    column_size_index: int = ...
    subheader_counts_index: int = ...
    column_text_index: int = ...
    column_name_index: int = ...
    column_attributes_index: int = ...
    format_and_label_index: int = ...
    column_list_index: int = ...
    data_subheader_index: int = ...

subheader_signature_to_index: Any
sas_date_formats: Any
sas_datetime_formats: Any



import numpy as np
from pandas.core.dtypes.cast import construct_1d_object_array_from_listlike as construct_1d_object_array_from_listlike, find_common_type as find_common_type, maybe_upcast_putmask as maybe_upcast_putmask
from pandas.core.dtypes.common import ensure_object as ensure_object, is_bool_dtype as is_bool_dtype, is_integer_dtype as is_integer_dtype, is_list_like as is_list_like, is_object_dtype as is_object_dtype, is_scalar as is_scalar
from pandas.core.dtypes.generic import ABCDatetimeArray as ABCDatetimeArray, ABCExtensionArray as ABCExtensionArray, ABCIndex as ABCIndex, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries, ABCTimedeltaArray as ABCTimedeltaArray
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.core.ops import missing as missing
from pandas.core.ops.dispatch import dispatch_to_extension_op as dispatch_to_extension_op, should_extension_dispatch as should_extension_dispatch
from pandas.core.ops.invalid import invalid_comparison as invalid_comparison
from pandas.core.ops.roperator import rpow as rpow
from typing import Any, Optional, Union

def comp_method_OBJECT_ARRAY(op: Any, x: Any, y: Any) -> Any: ...
def masked_arith_op(x: Any, y: Any, op: Any) -> Any: ...
def define_na_arithmetic_op(op: Any, str_rep: str) -> Any: ...
def na_arithmetic_op(left: Any, right: Any, op: Any, str_rep: str) -> Any: ...
def arithmetic_op(left: Union[np.ndarray, ABCExtensionArray], right: Any, op: Any, str_rep: str) -> Any: ...
def comparison_op(left: Union[np.ndarray, ABCExtensionArray], right: Any, op: Any) -> Union[np.ndarray, ABCExtensionArray]: ...
def na_logical_op(x: np.ndarray, y: Any, op: Any) -> Any: ...
def logical_op(left: Union[np.ndarray, ABCExtensionArray], right: Any, op: Any) -> Union[np.ndarray, ABCExtensionArray]: ...
def get_array_op(op: Any, str_rep: Optional[str]=...) -> Any: ...



from pandas.plotting._core import PlotAccessor as PlotAccessor, boxplot as boxplot, boxplot_frame as boxplot_frame, boxplot_frame_groupby as boxplot_frame_groupby, hist_frame as hist_frame, hist_series as hist_series
from pandas.plotting._misc import andrews_curves as andrews_curves, autocorrelation_plot as autocorrelation_plot, bootstrap_plot as bootstrap_plot, deregister as deregister_matplotlib_converters, lag_plot as lag_plot, parallel_coordinates as parallel_coordinates, plot_params as plot_params, radviz as radviz, register as register_matplotlib_converters, scatter_matrix as scatter_matrix, table as table



import abc
from pandas import DataFrame as DataFrame, Index as Index, Series as Series
from pandas.core.construction import create_series_with_explicit_dtype as create_series_with_explicit_dtype

from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.util._decorators import cache_readonly as cache_readonly
from typing import Any, Dict, Iterator, Tuple, Union

ResType = Dict[int, Any]

def frame_apply(obj: DataFrame, func: Any, axis: Any = ..., raw: bool=..., result_type: Any = ..., ignore_failures: bool=..., args: Any = ..., kwds: Any = ...) -> Any: ...

class FrameApply(metaclass=abc.ABCMeta):
    axis: int
    @property
    @abc.abstractmethod
    def result_index(self) -> Index: ...
    @property
    @abc.abstractmethod
    def result_columns(self) -> Index: ...
    @property
    @abc.abstractmethod
    def series_generator(self) -> Iterator[Series]: ...
    @abc.abstractmethod
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> Union[Series, DataFrame]: ...
    obj: Any = ...
    raw: Any = ...
    ignore_failures: Any = ...
    args: Any = ...
    kwds: Any = ...
    result_type: Any = ...
    f: Any = ...
    def __init__(self, obj: DataFrame, func: Any, raw: bool, result_type: Any, ignore_failures: bool, args: Any, kwds: Any): ...
    @property
    def res_columns(self) -> Index: ...
    @property
    def columns(self) -> Index: ...
    @property
    def index(self) -> Index: ...
    def values(self) -> Any: ...
    def dtypes(self) -> Series: ...
    @property
    def agg_axis(self) -> Index: ...
    def get_result(self) -> Any: ...
    def apply_empty_result(self) -> Any: ...
    def apply_raw(self) -> Any: ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    def apply_standard(self) -> Any: ...
    def apply_series_generator(self) -> Tuple[ResType, Index]: ...
    def wrap_results(self, results: ResType, res_index: Index) -> Union[Series, DataFrame]: ...

class FrameRowApply(FrameApply):
    axis: int = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    @property
    def series_generator(self) -> Any: ...
    @property
    def result_index(self) -> Index: ...
    @property
    def result_columns(self) -> Index: ...
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> DataFrame: ...

class FrameColumnApply(FrameApply):
    axis: int = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    @property
    def series_generator(self) -> Any: ...
    @property
    def result_index(self) -> Index: ...
    @property
    def result_columns(self) -> Index: ...
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> Union[Series, DataFrame]: ...
    def infer_to_same_shape(self, results: ResType, res_index: Index) -> DataFrame: ...



from typing import Any, Optional

def eval(expr: Any, parser: Any = ..., engine: Optional[str]=..., truediv: Any = ..., local_dict: Any = ..., global_dict: Any = ..., resolvers: Any = ..., level: Any = ..., target: Any = ..., inplace: Any = ...) -> Any: ...



import numpy as np
from pandas._typing import Dtype as Dtype
from pandas.core.dtypes.common import ensure_int16 as ensure_int16, ensure_int32 as ensure_int32, ensure_int64 as ensure_int64, ensure_int8 as ensure_int8, ensure_object as ensure_object, ensure_str as ensure_str, is_bool as is_bool, is_bool_dtype as is_bool_dtype, is_complex as is_complex, is_complex_dtype as is_complex_dtype, is_datetime64_dtype as is_datetime64_dtype, is_datetime64_ns_dtype as is_datetime64_ns_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_datetime_or_timedelta_dtype as is_datetime_or_timedelta_dtype, is_dtype_equal as is_dtype_equal, is_extension_array_dtype as is_extension_array_dtype, is_float as is_float, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_numeric_dtype as is_numeric_dtype, is_object_dtype as is_object_dtype, is_scalar as is_scalar, is_string_dtype as is_string_dtype, is_timedelta64_dtype as is_timedelta64_dtype, is_timedelta64_ns_dtype as is_timedelta64_ns_dtype, is_unsigned_integer_dtype as is_unsigned_integer_dtype, pandas_dtype as pandas_dtype
from pandas.core.dtypes.dtypes import DatetimeTZDtype as DatetimeTZDtype, ExtensionDtype as ExtensionDtype, IntervalDtype as IntervalDtype, PeriodDtype as PeriodDtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCDatetimeArray as ABCDatetimeArray, ABCDatetimeIndex as ABCDatetimeIndex, ABCPeriodArray as ABCPeriodArray, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_list_like as is_list_like
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.util._validators import validate_bool_kwarg as validate_bool_kwarg
from typing import Any, Optional

def maybe_convert_platform(values: Any) -> Any: ...
def is_nested_object(obj: Any) -> bool: ...
def maybe_downcast_to_dtype(result: Any, dtype: Any) -> Any: ...
def maybe_downcast_numeric(result: Any, dtype: Any, do_round: bool=...) -> Any: ...
def maybe_upcast_putmask(result: np.ndarray, mask: np.ndarray, other: Any) -> Any: ...
def maybe_promote(dtype: Any, fill_value: Any = ...) -> Any: ...
def infer_dtype_from(val: Any, pandas_dtype: bool=...) -> Any: ...
def infer_dtype_from_scalar(val: Any, pandas_dtype: bool=...) -> Any: ...
def infer_dtype_from_array(arr: Any, pandas_dtype: bool=...) -> Any: ...
def maybe_infer_dtype_type(element: Any) -> Any: ...
def maybe_upcast(values: Any, fill_value: Any = ..., dtype: Any = ..., copy: bool=...) -> Any: ...
def invalidate_string_dtypes(dtype_set: Any) -> None: ...
def coerce_indexer_dtype(indexer: Any, categories: Any) -> Any: ...
def coerce_to_dtypes(result: Any, dtypes: Any) -> Any: ...
def astype_nansafe(arr: Any, dtype: Any, copy: bool=..., skipna: bool=...) -> Any: ...
def maybe_convert_objects(values: np.ndarray, convert_numeric: bool=...) -> Any: ...
def soft_convert_objects(values: np.ndarray, datetime: bool=..., numeric: bool=..., timedelta: bool=..., coerce: bool=..., copy: bool=...) -> Any: ...
def convert_dtypes(input_array: Any, convert_string: bool=..., convert_integer: bool=..., convert_boolean: bool=...) -> Dtype: ...
def maybe_castable(arr: Any) -> bool: ...
def maybe_infer_to_datetimelike(value: Any, convert_dates: bool=...) -> Any: ...
def maybe_cast_to_datetime(value: Any, dtype: Any, errors: str=...) -> Any: ...
def find_common_type(types: Any) -> Any: ...
def cast_scalar_to_array(shape: Any, value: Any, dtype: Optional[Any] = ...) -> Any: ...
def construct_1d_arraylike_from_scalar(value: Any, length: int, dtype: Any) -> Any: ...
def construct_1d_object_array_from_listlike(values: Any) -> Any: ...
def construct_1d_ndarray_preserving_na(values: Any, dtype: Any = ..., copy: bool=...) -> Any: ...
def maybe_cast_to_integer_array(arr: Any, dtype: Any, copy: bool=...) -> Any: ...



import numpy as np
from pandas._libs.tslibs import  resolution as resolution
from pandas.core.accessor import delegate_names as delegate_names
from pandas.core.arrays.period import PeriodArray as PeriodArray, period_array as period_array, raise_on_incompatible as raise_on_incompatible, validate_dtype_freq as validate_dtype_freq
from pandas.core.dtypes.common import ensure_platform_int as ensure_platform_int, is_bool_dtype as is_bool_dtype, is_datetime64_any_dtype as is_datetime64_any_dtype, is_dtype_equal as is_dtype_equal, is_float as is_float, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_object_dtype as is_object_dtype, pandas_dtype as pandas_dtype
from pandas.core.indexes.base import ensure_index as ensure_index, maybe_extract_name as maybe_extract_name
from pandas.core.indexes.datetimelike import DatetimeIndexOpsMixin as DatetimeIndexOpsMixin, DatetimelikeDelegateMixin as DatetimelikeDelegateMixin
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex, Index as Index
from pandas.core.indexes.numeric import Int64Index as Int64Index
from pandas.core.missing import isna as isna
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.tseries import frequencies as frequencies
from pandas.tseries.offsets import DateOffset as DateOffset, Tick as Tick
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from typing import Any, Optional

class PeriodDelegateMixin(DatetimelikeDelegateMixin): ...

class PeriodIndex(DatetimeIndexOpsMixin, Int64Index, PeriodDelegateMixin):
    def __new__(cls, data: Optional[Any] = ..., ordinal: Optional[Any] = ..., freq: Optional[Any] = ..., tz: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ..., name: Optional[Any] = ..., **fields: Any) -> Any: ...
    @property
    def values(self) -> Any: ...
    def __contains__(self, key: Any) -> bool: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __array_wrap__(self, result: Any, context: Optional[Any] = ...) -> Any: ...
    def asof_locs(self, where: Any, mask: Any) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ..., how: str = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    @property
    def is_full(self) -> bool: ...
    @property
    def inferred_type(self) -> str: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def get_indexer(self, target: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer_non_unique(self, target: Any) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def insert(self, loc: Any, item: Any) -> Any: ...
    def join(self, other: Any, how: str = ..., level: Optional[Any] = ..., return_indexers: bool = ..., sort: bool = ...) -> Any: ...
    def intersection(self, other: Any, sort: bool = ...) -> Any: ...
    def difference(self, other: Any, sort: Optional[Any] = ...) -> Any: ...
    def memory_usage(self, deep: bool = ...) -> Any: ...

def period_range(start: Any = ..., end: Any = ..., periods: Any = ..., freq: Any = ..., name: Any = ...) -> PeriodIndex: ...



from typing import Any

def get_group_index(labels: Any, shape: Any, sort: bool, xnull: bool) -> Any: ...
def get_compressed_ids(labels: Any, sizes: Any) -> Any: ...
def is_int64_overflow_possible(shape: Any) -> bool: ...
def decons_group_index(comp_labels: Any, shape: Any) -> Any: ...
def decons_obs_group_ids(comp_ids: Any, obs_ids: Any, shape: Any, labels: Any, xnull: bool) -> Any: ...
def indexer_from_factorized(labels: Any, shape: Any, compress: bool=...) -> Any: ...
def lexsort_indexer(keys: Any, orders: Any = ..., na_position: str=...) -> Any: ...
def nargsort(items: Any, kind: str=..., ascending: bool=..., na_position: str=...) -> Any: ...

class _KeyMapper:
    levels: Any = ...
    labels: Any = ...
    comp_ids: Any = ...
    k: Any = ...
    tables: Any = ...
    def __init__(self, comp_ids: Any, ngroups: int, levels: Any, labels: Any) -> None: ...
    def get_key(self, comp_id: Any) -> Any: ...

def get_flattened_iterator(comp_ids: Any, ngroups: Any, levels: Any, labels: Any) -> Any: ...
def get_indexer_dict(label_list: Any, keys: Any) -> Any: ...
def get_group_index_sorter(group_index: Any, ngroups: int) -> Any: ...
def compress_group_index(group_index: Any, sort: bool=...) -> Any: ...



from __future__ import annotations

import datetime
import sys
from decimal import Decimal
from fractions import Fraction
from io import RawIOBase, BufferedIOBase, TextIOBase, TextIOWrapper
from mmap import mmap
from os import PathLike
from numbers import Number

from pandas.core.arrays.base import ExtensionArray as ExtensionArray
from pandas.core.indexes.base import Index as Index
from typing import Any, AnyStr, Callable, Collection, Dict, Hashable, IO, List, Mapping, Optional, TypeVar, Union, \
    ByteString, Pattern

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

import numpy as np

from pandas import Interval as Interval, tseries
from pandas._libs import Period, Timedelta, Timestamp
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.series import Series as Series
from pandas.tseries.offsets import DateOffset

# array-like
ArrayLike = Union[ExtensionArray, np.ndarray]
AnyArrayLike = Union[ArrayLike, 'Index', 'Series']

# scalar
PythonScalar = Union[str, int, float, bool]
DatetimeLikeScalar = Union[Period, Timestamp, Timedelta]
ExtraNumpyScalar = Union[np.object, np.object_, np.bytes, np.bytes_, np.unicode, np.unicode_, np.void]
NumpyBooleans = Union[np.bool, np.bool_, np.bool8, np.ScalarType]
NumpyIntegers = Union[np.byte, np.char, np.intc, np.int, np.int_, np.longlong, np.intp, np.int8, np.int16, np.int32, np.int64]
NumpyUnsignedIntegers = Union[np.ubyte, np.ushort, np.unintc, np.uint, np.ulonglong, np.uintp, np.uint8, np.uint16, np.uint32, np.uint64]
NumpyFloats = Union[np.half, np.double, np.float, np.float_, np.longfloat, np.float16, np.float32, np.float64, np.float128]
NumpyComplex = Union[np.csingle, np.complex, np.complex_, np.clongfloat, np.complex64, np.complex128, np.complex192, np.complex256]
NumpyScalar = Union[NumpyBooleans, NumpyIntegers, NumpyUnsignedIntegers, NumpyFloats, NumpyComplex]
PandasScalar = Union[Period, Timestamp, Timedelta, Interval]
Scalar = Union[PythonScalar, NumpyScalar, Decimal, ByteString, Fraction, DateOffset, Interval, Number, datetime.datetime, datetime.timedelta]

TimestampConvertible = Union[DatetimeLikeScalar, datetime.date, str, int, float]

Orientation = Literal['index', 'columns']
AxisOption = Union[Literal[0, 1], Orientation]
OneDimensionalAxisOption = Literal[0, 'index']
ReplaceMethod = Literal['linear', 'time', 'index', 'values', 'pad', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic', 'spline',
              'barycentric', 'polynomial', 'krogh', 'piecewise_polynomial', 'spline', 'pchip', 'akima', 'from_derivatives']
SortKind = Literal['quicksort', 'mergesort', 'heapsort']
NoneNumpyCompatible = Literal[None] # this is sometimes used to ensure Numpy compatibility
GeneralDuplicatesKeepStrategy = Literal['first', 'last', False]
InterpolationMethod = Literal['linear', 'lower', 'higher', 'midpoint', 'nearest']
NamedCorrelationMethod = Literal['pearson', 'kendall', 'spearman']
SearchSide = Literal['left', 'right']
NaSortPosition = Literal['first', 'last']
JoinType = Literal['left', 'right', 'outer', 'inner']
FillMethod = Literal['backfill', 'bfill', 'pad', 'ffill']
ErrorsStrategy = Literal['ignore', 'raise']
TimestampMethod = Literal['s', 'e', 'start', 'end']

UserCorrelationMethod = Callable[[np.ndarray, np.ndarray], Scalar]
CorrelationMethod = Union[NamedCorrelationMethod, UserCorrelationMethod]

Dtype: Any
T = TypeVar('T')

Buffer = Union[IO[AnyStr], RawIOBase, BufferedIOBase, TextIOBase, TextIOWrapper, mmap]
FileOrBuffer = Union[str, Buffer[T]]
FilePathOrBuffer = Union["PathLike[str]", FileOrBuffer[T]]

FrameOrSeriesUnion: Union[DataFrame, Series]
FrameOrSeries = Union[DataFrame, Series]
Axis = Union[str, int]
Label = Optional[Hashable]
Level = Union[Label, int]
Ordered = Optional[bool]
JSONSerializable = Union[PythonScalar, List, Dict]
Axes = Collection
Renamer = Union[Mapping[Label, Any], Callable[[Label], Label]]

FillValue = Union[Scalar, Dict[Any, Any], FrameOrSeries]

# Any plain Python or numpy function
Function = Union[np.func, Callable[..., Any]]
# Also including function names e.g. np.exp. 'sqrt'
FunctionOrName = Union[Function, str]
# Used in SelectionMixin and inheriting classes
AggregationFunction = Union[FunctionOrName, List[FunctionOrName], Dict[Axis, Union[FunctionOrName, List[FunctionOrName]]]]

F = TypeVar("F", bound=Function)

Column = Union[int, str]


# types of vectorized key functions for DataFrame::sort_values and
# DataFrame::sort_index, among others
ValueKeyFunc = Optional[Callable[["Series"], Union["Series", AnyArrayLike]]]
IndexKeyFunc = Optional[Callable[["Index"], Union["Index", AnyArrayLike]]]

# Actually google.auth.credentials.Credentials
GoogleCredentials = Any

# Replacement
RegexElement = Union[str, Pattern]
RegexArgument = Union[RegexElement, List[RegexElement], Dict[Column, RegexElement], Dict[Column, Dict[RegexElement, RegexElement]]]
ToReplace = Union[Scalar, Dict[Column, Scalar], Dict[Column, Dict[Scalar, Scalar]], List[Scalar], RegexArgument]
ReplaceValue = Union[Scalar, Dict[Column, Scalar], List[Scalar], RegexArgument]


Frequency = Union[DateOffset, tseries.offsets.liboffsets.BaseOffset, datetime.timedelta, str]

GroupByObject = Union[Label, List[Label], Function, Series, np.ndarray, Mapping[Label, Any]]



from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from typing import Any

def unpack_zerodim_and_defer(name: str) -> Any: ...



from typing import Any

is_bool: Any
is_integer: Any
is_float: Any
is_complex: Any
is_scalar: Any
is_decimal: Any
is_interval: Any
is_list_like: Any

def is_number(obj: Any) -> bool: ...
def is_iterator(obj: Any) -> bool: ...
def is_file_like(obj: Any) -> bool: ...
def is_re(obj: Any) -> bool: ...
def is_re_compilable(obj: Any) -> bool: ...
def is_array_like(obj: Any) -> bool: ...
def is_nested_list_like(obj: Any) -> bool: ...
def is_dict_like(obj: Any) -> bool: ...
def is_named_tuple(obj: Any) -> bool: ...
def is_hashable(obj: Any) -> bool: ...
def is_sequence(obj: Any) -> bool: ...



from datetime import datetime

from pandas._libs.tslibs import offsets as liboffsets
from pandas._libs.tslibs.offsets import BaseOffset
from typing import Any, Optional, Sequence, Dict


class DateOffset(BaseOffset):
    normalize: bool = ...
    def __init__(self, n: int = ..., normalize: bool = ..., **kwds: Any) -> None: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> Any: ...
    def is_anchored(self) -> Any: ...
    def onOffset(self, dt: Any) -> Any: ...
    def isAnchored(self) -> Any: ...
    @property
    def name(self) -> Any: ...
    def rollback(self, dt: Any) -> Any: ...
    def rollforward(self, dt: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    @property
    def rule_code(self) -> Any: ...
    def freqstr(self) -> Any: ...
    @property
    def nanos(self) -> None: ...

class SingleConstructorOffset(DateOffset): ...

class _CustomMixin:
    def __init__(self, weekmask: Any, holidays: Any, calendar: Any) -> None: ...

class BusinessMixin:
    @property
    def offset(self) -> Any: ...

class BusinessDay(BusinessMixin, SingleConstructorOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., offset: Any = ...) -> None: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...

class BusinessHourMixin(BusinessMixin):
    def __init__(self, start: str = ..., end: str = ..., offset: Any = ...) -> None: ...
    def next_bday(self) -> Any: ...
    def rollback(self, dt: Any) -> Any: ...
    def rollforward(self, dt: Any) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...

class BusinessHour(BusinessHourMixin, SingleConstructorOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., start: str = ..., end: str = ..., offset: Any = ...) -> None: ...

class CustomBusinessDay(_CustomMixin, BusinessDay):
    def __init__(self, n: int = ..., normalize: bool = ..., weekmask: str = ..., holidays: Optional[Any] = ..., calendar: Optional[Any] = ..., offset: Any = ...) -> None: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> None: ...
    def is_on_offset(self, dt: Any) -> Any: ...

class CustomBusinessHour(_CustomMixin, BusinessHourMixin, SingleConstructorOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., weekmask: str = ..., holidays: Optional[Any] = ..., calendar: Optional[Any] = ..., start: str = ..., end: str = ..., offset: Any = ...) -> None: ...

class MonthOffset(SingleConstructorOffset):
    __init__: Any = ...
    @property
    def name(self) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> Any: ...

class MonthEnd(MonthOffset): ...
class MonthBegin(MonthOffset): ...
class BusinessMonthEnd(MonthOffset): ...
class BusinessMonthBegin(MonthOffset): ...

class _CustomBusinessMonth(_CustomMixin, BusinessMixin, MonthOffset):
    is_on_offset: Any = ...
    apply_index: Any = ...
    def __init__(self, n: int = ..., normalize: bool = ..., weekmask: str = ..., holidays: Optional[Any] = ..., calendar: Optional[Any] = ..., offset: Any = ...) -> None: ...
    def cbday_roll(self) -> Any: ...
    def m_offset(self) -> Any: ...
    def month_roll(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...

class CustomBusinessMonthEnd(_CustomBusinessMonth): ...
class CustomBusinessMonthBegin(_CustomBusinessMonth): ...

class SemiMonthOffset(DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., day_of_month: Optional[Any] = ...) -> None: ...
    @property
    def rule_code(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> Any: ...

class SemiMonthEnd(SemiMonthOffset):
    def is_on_offset(self, dt: Any) -> Any: ...

class SemiMonthBegin(SemiMonthOffset):
    def is_on_offset(self, dt: Any) -> Any: ...

class Week(DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., weekday: Optional[Any] = ...) -> None: ...
    def is_anchored(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, i: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    @property
    def rule_code(self) -> Any: ...

class _WeekOfMonthMixin:
    def apply(self, other: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...

class WeekOfMonth(_WeekOfMonthMixin, DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., week: int = ..., weekday: int = ...) -> None: ...
    @property
    def rule_code(self) -> Any: ...

class LastWeekOfMonth(_WeekOfMonthMixin, DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., weekday: int = ...) -> None: ...
    @property
    def rule_code(self) -> Any: ...

class QuarterOffset(DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., startingMonth: Optional[Any] = ...) -> None: ...
    def is_anchored(self) -> Any: ...
    @property
    def rule_code(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    def apply_index(self, dtindex: Any) -> Any: ...

class BQuarterEnd(QuarterOffset): ...
class BQuarterBegin(QuarterOffset): ...
class QuarterEnd(QuarterOffset): ...
class QuarterBegin(QuarterOffset): ...

class YearOffset(DateOffset):
    def apply(self, other: Any) -> Any: ...
    def apply_index(self, dtindex: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    def __init__(self, n: int = ..., normalize: bool = ..., month: Optional[Any] = ...) -> None: ...
    @property
    def rule_code(self) -> Any: ...

class BYearEnd(YearOffset): ...
class BYearBegin(YearOffset): ...
class YearEnd(YearOffset): ...
class YearBegin(YearOffset): ...

class FY5253(DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., weekday: int = ..., startingMonth: int = ..., variation: str = ...) -> None: ...
    def is_anchored(self) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def get_year_end(self, dt: Any) -> Any: ...
    @property
    def rule_code(self) -> Any: ...
    def get_rule_code_suffix(self) -> Any: ...

class FY5253Quarter(DateOffset):
    def __init__(self, n: int = ..., normalize: bool = ..., weekday: int = ..., startingMonth: int = ..., qtr_with_extra_week: int = ..., variation: str = ...) -> None: ...
    def is_anchored(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def get_weeks(self, dt: Any) -> Any: ...
    def year_has_extra_week(self, dt: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...
    @property
    def rule_code(self) -> Any: ...

class Easter(DateOffset):
    __init__: Any = ...
    def apply(self, other: Any) -> Any: ...
    def is_on_offset(self, dt: Any) -> Any: ...

class Tick(liboffsets._Tick, SingleConstructorOffset):
    def __init__(self, n: int = ..., normalize: bool = ...) -> None: ...
    __gt__: Any = ...
    __ge__: Any = ...
    __lt__: Any = ...
    __le__: Any = ...
    def __add__(self, other: Any) -> Any: ...
    def __eq__(self, other: Any) -> bool: ...
    def __hash__(self) -> Any: ...
    def __ne__(self, other: Any) -> Any: ...
    @property
    def delta(self) -> Any: ...
    @property
    def nanos(self) -> Any: ...
    def apply(self, other: Any) -> Any: ...
    def is_anchored(self) -> Any: ...

class Day(Tick): ...
class Hour(Tick): ...
class Minute(Tick): ...
class Second(Tick): ...
class Milli(Tick): ...
class Micro(Tick): ...
class Nano(Tick): ...
BDay = BusinessDay
BMonthEnd = BusinessMonthEnd
BMonthBegin = BusinessMonthBegin
CBMonthEnd = CustomBusinessMonthEnd
CBMonthBegin = CustomBusinessMonthBegin
CDay = CustomBusinessDay

def generate_range(start: datetime, end: datetime, periods: int, offset: Any) -> Sequence[Any]: ...

prefix_mapping: Dict[str, str]


from typing import Any

import numpy as np
from pandas._typing import AnyArrayLike as AnyArrayLike
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries

def is_list_like_indexer(key: Any) -> bool: ...
def is_scalar_indexer(indexer: Any, arr_value: Any) -> bool: ...
def is_empty_indexer(indexer: Any, arr_value: np.ndarray) -> bool: ...
def check_setitem_lengths(indexer: Any, value: Any, values: Any) -> None: ...
def validate_indices(indices: np.ndarray, n: int) -> None: ...
def maybe_convert_indices(indices: Any, n: int) -> Any: ...
def length_of_indexer(indexer: Any, target: Any = ...) -> int: ...
def deprecate_ndim_indexing(result: Any) -> None: ...
def check_array_indexer(array: AnyArrayLike, indexer: Any) -> Any: ...



from pandas.core.accessor import PandasDelegate as PandasDelegate, delegate_names as delegate_names
from pandas.core.arrays import DatetimeArray as DatetimeArray, PeriodArray as PeriodArray, TimedeltaArray as TimedeltaArray
from pandas.core.base import NoNewAttributesMixin as NoNewAttributesMixin, PandasObject as PandasObject

from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.core.indexes.datetimes import DatetimeIndex as DatetimeIndex
from pandas.core.indexes.timedeltas import TimedeltaIndex as TimedeltaIndex
from typing import Any

class Properties(PandasDelegate, PandasObject, NoNewAttributesMixin):
    orig: Any = ...
    name: Any = ...
    def __init__(self, data: Any, orig: Any) -> None: ...

class DatetimeProperties(Properties):
    def to_pydatetime(self) -> Any: ...
    @property
    def freq(self) -> Any: ...

class TimedeltaProperties(Properties):
    def to_pytimedelta(self) -> Any: ...
    @property
    def components(self) -> Any: ...
    @property
    def freq(self) -> Any: ...

class PeriodProperties(Properties): ...

class CombinedDatetimelikeProperties(DatetimeProperties, TimedeltaProperties, PeriodProperties):
    def __new__(cls, data: Any) -> Any: ...



from pandas import DataFrame as DataFrame, Index as Index, MultiIndex as MultiIndex, Series as Series
from pandas._typing import Axis, FrameOrSeries, Label
from pandas.core.arrays.categorical import factorize_from_iterable as factorize_from_iterable, factorize_from_iterables as factorize_from_iterables
from pandas.core.generic import NDFrame as NDFrame
from pandas.core.indexes.api import all_indexes_same as all_indexes_same, ensure_index as ensure_index, get_consensus_names as get_consensus_names, get_objs_combined_axis as get_objs_combined_axis
from pandas.core.internals import concatenate_block_managers as concatenate_block_managers
from typing import Any, Iterable, Mapping, overload

# For some reason mypy won't read this declaration correctly if we use 2 overloads with union
@overload
def concat(objs: Iterable[DataFrame], axis: Axis = ..., join: str=..., ignore_index: bool=..., keys: Any = ..., levels: Any = ..., names: Any = ..., verify_integrity: bool=..., sort: bool=..., copy: bool=...) -> DataFrame: ...
@overload
def concat(objs: Mapping[Label, DataFrame], axis: Axis = ..., join: str=..., ignore_index: bool=..., keys: Any = ..., levels: Any = ..., names: Any = ..., verify_integrity: bool=..., sort: bool=..., copy: bool=...) -> DataFrame: ...
@overload
def concat(objs: Iterable[Series], axis: Axis = ..., join: str=..., ignore_index: bool=..., keys: Any = ..., levels: Any = ..., names: Any = ..., verify_integrity: bool=..., sort: bool=..., copy: bool=...) -> FrameOrSeries: ...
@overload
def concat(objs: Mapping[Label, Series], axis: Axis = ..., join: str=..., ignore_index: bool=..., keys: Any = ..., levels: Any = ..., names: Any = ..., verify_integrity: bool=..., sort: bool=..., copy: bool=...) -> FrameOrSeries: ...

class _Concatenator:
    intersect: bool = ...
    objs: Any = ...
    axis: Any = ...
    keys: Any = ...
    names: Any = ...
    levels: Any = ...
    sort: Any = ...
    ignore_index: Any = ...
    verify_integrity: Any = ...
    copy: Any = ...
    new_axes: Any = ...
    def __init__(self, objs: Any, axis: Any = ..., join: str=..., keys: Any = ..., levels: Any = ..., names: Any = ..., ignore_index: bool=..., verify_integrity: bool=..., copy: bool=..., sort: Any = ...) -> None: ...
    def get_result(self) -> Any: ...
